;;;; e-macs.el ---  Routines specific to Emacs but not XEmacs for every OS.
;;-*- Mode: Emacs-Lisp -*-   ;Comment that puts emacs into lisp mode
;;Time-stamp: <2002-05-20 11:52:53 bingalls>
;;;;Outline-Mode: C-c @ C-t to show as index. C-c @ C-a to show all.
;;$Id$

;;__________________________________________________________________________
;;;;	TABLE OF CONTENTS
;;	Legal
;;	Commentary
;;	Code
;;	Display & Menu
;;		Colors
;;		Fonts
;;		Keys
;;		Menu Customization
;;		Minibuffer Scrolling
;;		Minibuffer (status area) Sizing
;;		Recent Files Menu
;;	Utilities
;;		Emacs Server
;;		File Handling
;;		split-path()
;;		Toggle Text Mode
;;		Unibyte Support
;;		yank-line()
;;	Internet
;;		Email To Address At Cursor
;;		Remote Editing & Ftp Interface
;;		Web Browsing
;;	Applications
;;		Babel Translation
;;		Telephone/Address Book
;;	Programming
;;		Emacs Lisp
;;		Function Menu
;;		Tags: Find C Declarations & Refs Fast
;;		Outline-mode control variables

;;__________________________________________________________________________
;;; Legal:
;;Copyright  1998,2002 Bruce Ingalls bingalls@sf.net
;;This file is not (yet) part of Emacs nor XEmacs. See file COPYING for license

;;__________________________________________________________________________
;;; Commentary:
;;Emacs editor startup file.  Routines for every OS, but only Gnu Emacs.

;;__________________________________________________________________________
;;; Author:		Bruce Ingalls <bingalls@panix.com>
;;<url:http://www.sourceforge.net/projects/emacro>
;;<url:http://www.dotemacs.de/>
;;; Change Log:		See ChangeLog history file
;;; Keywords:		gnu

;;__________________________________________________________________________
;;;; Code:
(autoload 'outline-minor-mode "outline" "automagic index to jump to" t)
(defgroup e-macs nil  "Settings from e-macs.el file."  :group 'emacro)

;;__________________________________________________________________________
;;;;    DISPLAY & MENU
;;(menu-bar-mode nil)                    ;turn off menu bar
;;(load-library "term/pc-win")

(cache-locate-library
 use-cache 'cache-mic-paren "mic-paren"
 "For Gnu Emacs parenthesis, brace, etc., matching, get mic-paren.el from <url:http://www.docs.uu.se/~mic/emacs.html>")

(cond (cache-mic-paren
       (require 'mic-paren)
       (defcustom paren-face 'bold "Style of hilight."
	 :group 'e-macs
	 :type 'string)
       (defcustom paren-sexp-mode t
	 "Non-nil enables highlight of matching paren, only."
	 :group 'e-macs
	 :type 'boolean)
       (show-paren-mode 1))
      (t (progn
	   (require 'paren)
	   (show-paren-mode 1))))       ;highlight matching parenthesis

(when (not (eq system-type 'darwin))	;OS X console hates scrollbars
  (defcustom e-scrollbar-side 'right "Side where scrollbar appears."
    :group 'e-macs
    :type '(choice (const :tag "none (nil)")
		   (const left)
		   (const right))
    :set 'set-scroll-bar-mode-1)
  (setq scroll-bar-mode 'e-scrollbar-side))
;;(toggle-scroll-bar t)			;refresh scroll bar

(require 'msb)                          ;Mouse Select Buffer: popup menus
;;(menu-bar-mode (if window-system 1 -1))	;Disable menu-bar in console

;;__________________________________________________________________________
;;;;;;		Colors
;;See also Programmer.el
;;(set-background-color "grey90")
(defcustom global-font-lock-mode t "Syntax colorize all supported modes."
  :group 'e-macs
  :type 'boolean)

;;__________________________________________________________________________
;;;;;;		Keys
;;(require 'pc-mode)			;similar code is in keys.el
(require 'pc-select)                    ;Shift-Arrow highlighting
(pc-selection-mode)

(defvar cache-cua)		;shut up compiler
(cache-locate-library
 use-cache 'cache-cua "cua"
 "For CUA key support, get cua.el from <url:http://www.cua.dk/cua.html>")

(when cache-cua
  (require 'cua)
  (CUA-mode t)
  ;;(CUA-keypad-mode 'prefix t)		;saves into numbered registers
  )

;;Next 2 lines may cause error:
;; "Symbol's value as variable is void: minibuffer-local-ns-map"
(require 'delsel)                       ;delete selected text when overtyped
(delete-selection-mode t)               ;Overwrites selected text

(require 'mouse-sel)

;;override one of the libraries above
(global-set-key [(meta backspace)]      'backward-kill-word)
(global-set-key [(control backspace)]   'undo)

;;__________________________________________________________________________
;;;;;;		Fonts
;;(set *default-font* "-adobe-courier-bold-*-*-*-20-*-*-*-*-*-*-*")
;;'xfontsel' in Linux lists fonts. Here's how to set a default for emacs:
;;(set-default-font "-*-Lucida Console-normal-r-*-*-11-82-*-*-c-*-*-#204-")
;;(set-default-font "-adobe-courier-*-*-*-14-*-*-*-*-*-*-*")

;;__________________________________________________________________________
;;;;;;		Menu Customization
(defvar emacro-version)			;shut up compiler
(define-key global-map [menu-bar help-menu emacro-help]
  `(,(concat "EMacro v" emacro-version) . emacro-help))

(define-key global-map [menu-bar tools locate] '("Locate file" . locate))
(define-key global-map [menu-bar tools web] '("W3 web browser" . w3))
(define-key global-map [menu-bar tools Refresh_colors]
  '("Refresh colors" . font-lock-fontify-buffer))

;;turn off emacs toolbar, Emacs v21
;;(tool-bar-mode -1)

(cache-locate-library
 use-cache 'cache-a2ps-print "a2ps-print"
 "For clean source code printouts, get a2ps-print.el from <url: ftp://ftp.enst.fr/pub/unix/a2ps/ >")

(when cache-a2ps-print
  (autoload 'a2ps-region "a2ps-print" "a2ps-print" t)
  (autoload 'a2ps-buffer "a2ps-print" "a2ps-print" t)
  (define-key function-key-map [f22] [print])
  (global-set-key [print] 'a2ps-buffer) ;f22 is Print Screen
  (global-set-key [(shift print)] 'a2ps-region)	;print selected text
  (define-key global-map [menu-bar files a2ps-buffer]
    '("a2ps Print Buffer" . a2ps-buffer)))

;;Remote printing example to put into postload.el:
;;(setq a2ps-command "ssh")
;;(setq a2ps-switches '("myHost" "a2ps" "-P" "myPSprinter" "--line-numbers=1"))

;;If you don't have postscript nor ghostscript:
;;(define-key global-map [menu-bar files print-buf]
;;  '("Print Buffer" . print-buffer))
;;(load-library "ps-print")      ;see also a2ps
;;You could use ps-spool-... instead of ps-print-...; then call 'ps-despool'
;; to batch up print jobs
;;(define-key global-map [menu-bar files ps-print]
;;                       '("PS Print Buffer" . ps-print-buffer-with-faces))
;;(define-key global-map [menu-bar files print_selection]
;;                       '("PS Print Selection" . ps-print-region-with-faces))
;;(define-key global-map [menu-bar files] nil)    ;disable above menu
;;In unix, take a look at /etc/printcap for printer definitions

(define-key global-map [menu-bar macros macro5]
  '("Assign Key to Named Macro". global-set-key))

(define-key global-map [menu-bar macros macro4]
  '("Name Last Macro" . name-last-kbd-macro))

(define-key global-map [menu-bar macros macro3]
  '("Execute Last Macro" . call-last-kbd-macro)) 

(define-key global-map [menu-bar macros macro2]
  '("Stop Macro Recording" . end-kbd-macro))

(define-key global-map [menu-bar macros macro1]
  '("Start Macro Recording" . start-kbd-macro))

;;__________________________________________________________________________
;;;;;;		Recent Files Menu
(cache-locate-library
 use-cache 'cache-recentf "recentf"
 "To get recently opened files on menu, get recentf from <url:http://perso.wanadoo.fr/david.ponce/more-elisp.html>")

;;(autoload 'recentf-mode "recentf" "recent files list" t) ;shut up compiler


(when (string-match "Gnu" (emacs-version))
  (when (and cache-recentf window-system)
    (require 'recentf)
    (recentf-mode t)))

;;__________________________________________________________________________
;;;;;;		Minibuffer Scrolling
;;This causes XEmacs to lock up, ex: C-x k won't respond to kill buffer
;;(cache-locate-library use-cache 'cache-mscroll "mscroll"
;; "For a minibuffer that scrolls, get mscroll.el from <url:http://vegemite.chem.nott.ac.uk/~matt/emacs/>")

;;(cond
;; (cache-mscroll
;;  (require 'mscroll)
;;  (mscroll-activate)
;;  (add-to-list 'mscroll-ignored-messages "^Compiling .*\\.el")))

;;__________________________________________________________________________
;;;;;;		Minibuffer (status area) Sizing
;;resize-minibuffer-mode auto resizes the minibuffer to hold its contents
;;bundled with newer emacs?
;;(require 'rsz-mini)
;;(autoload 'resize-minibuffer-mode "rsz-minibuf" nil t)
;;(resize-minibuffer-mode)
;;(setq resize-minibuffer-window-exactly nil)

;;__________________________________________________________________________
;;;; UTILITIES

;;__________________________________________________________________________
;;;;;;		Emacs Server
(defvar server-process)                 ;shut up bytecompiler

(autoload 'gnuserv-running-p "gnuserv" "gnuserv." t) ;shut up compiler
(autoload 'gnuserv-start "gnuserv" "gnuserv." t) ;shut up compiler
(if (which "gnuclientw")		;w32 emacs client
;; (string-match "windows" (symbol-name system-type))
  (progn
    (require 'gnuserv)
    (setq server-done-function 'bury-buffer gnuserv-frame (car (frame-list)))
    (gnuserv-start)
;;open buffer in existing frame instead of creating a new one...
    (setq gnuserv-frame (selected-frame)))

;;else use bundled emacs server
  (when
      (not (or (string-match "windows" (symbol-name system-type))
	       (string-match "macos" (symbol-name system-type))))
    (require 'server)
    (if (and server-process
	     (equal (process-status server-process) 'run))
	(progn
;;New 'window' when loading new files:
;;  (add-hook 'server-visit-hook #'(lambda() (setq server-window (new-frame))))

	  (global-set-key [(control xc)]	'delete-frame)
	  (global-set-key [(meta f4)]	'delete-frame))
      ;;else this is the only emacs instance:
      (server-start))))


;;THIS REWORKED CODE FAILS FOR SOME W98 USERS?
;;Your second emacs session shares memory to start up faster
;; (defvar gnuserv)			;shut up compiler
;; (cache-locate-library use-cache 'cache-gnuserv "gnuserv"
;; 		      "For faster restarts, get gnuserv.el")
;; (defvar server)			;shut up compiler
;; (cache-locate-library use-cache 'cache-server "server"
;; 		      "For faster restarts, get server.el")

;; (cond
;; ;;Some install XEmacs's gnuserv for its remote capabilities
;;  ((and cache-gnuserv (which "gnuserv"))
;;   (autoload 'gnuserv-running-p "gnuserv" "gnuserv." t) ;shut up compiler
;;   (autoload 'gnuserv-start "gnuserv" "gnuserv." t) ;shut up compiler
;; ;;  (setenv "GNUSERV_SHOW_EMACS" "1")	;force emacs on top
;;   (if (not (featurep 'gnuserv))
;;       (progn
;; 	(require 'gnuserv)
;; 	(gnuserv-start))

;; ;;Else new 'window' when loading new files:
;; ;;  (add-hook 'server-visit-hook #'(lambda() (setq server-window (new-frame))))
;; ;;      (if (and (fboundp gnuserv-running-p) (gnuserv-running-p))

;;     (progn
;;       (global-unset-key [(control xc)])
;;       (global-unset-key [(meta f4)])
;;       (global-set-key [(control xc)] 'delete-frame)
;;       (global-set-key [(meta f4)] 'delete-frame)
;; ;;else this is the only emacs instance:
;; ;;      (gnuserv-start)
;;       )))
;; ;;Else use server, which is bundled with Emacs
;;  ((and cache-server (which "emacsserver"))
;;   (require 'server)
;;   (autoload 'server-edit "server" "server." t) ;shut up compiler
;;   (defvar server-process)                 ;shut up bytecompiler
;;   (if (and server-process (equal (process-status server-process) 'run))
;;       (progn
;; ;;New 'window' when loading new files:
;; ;;  (add-hook 'server-visit-hook #'(lambda() (setq server-window (new-frame))))

;; 	(global-unset-key [(control xc)])
;; 	(global-unset-key [(meta f4)])
;; 	(global-set-key [(control xc)] (lambda() (server-edit "done")))
;; 	(global-set-key [(meta f4)] (lambda() (server-edit "done"))))
;;     ;;else this is the only emacs instance:
;;     (server-start))))

;;__________________________________________________________________________
;;;;;;		File Handling
;;<url:ftp://ls6-ftp.cs.uni-dortmund.de/pub/src/emacs/>
;;This sometimes works in xemacs
(autoload 'locate "locate" "Quickly find unix files." t)
(autoload 'locate-with-filter "locate" "filter output of locate")

;;__________________________________________________________________________
;;;;;;		Unibyte Support
;;(set-language-environment "Latin-1")
;;(set-terminal-coding-system 'iso-latin-1)
;;(set-terminal-coding-system 'iso-8859-1)
;;(setq unibyte-display-via-language-environment t)

;;__________________________________________________________________________
;;;;;;		Toggle Text Mode
(toggle-text-mode-auto-fill);;hook for filling in text & la/tex mode

;;__________________________________________________________________________
;;;;;;		yank-line
(defun yank-line () "copies current line. Should be merged with XEmacs ver"
  (interactive)
  (push-mark (beginning-of-line))
  (end-of-line)
  (copy-region-as-kill (mark) (point)))

;;__________________________________________________________________________
;;;; INTERNET
;;__________________________________________________________________________
;;;;;;		Email To Address At Cursor
;; Based on a defun by Kevin Rodgers <kevinr@ihs.com>
;;(require 'thingatpt)
(autoload 'thing-at-point "thingatpt" "mouse enable object" t)

(defvar email-regexp)
(defun mail-to-address-at-point ()
  "*Edit a new mail message to the address at point."
  (interactive)
  (let ((url-at-point (substring (thing-at-point 'url) 7)))
    (if (string-match email-regexp url-at-point)
        (mail nil url-at-point)
      (message "Bad email address"))))

;;__________________________________________________________________________
;;;;;;		Remote Editing & FTP Interface
;;Open with     C-x C-f [ret] /username@host:/path/file.name
;;ange-ftp should load on demand without needing this line, but it helps me.
;;(require 'ange-ftp)
(autoload 'ange-ftp "ange-ftp" "ange-ftp edit remote files." t)
(defvar use-login-name)			;shut up compiler
(defconst ange-ftp-default-user use-login-name)

(setq default-directory (expand-file-name "./")) ;not needed, but may help

(defcustom backup-by-copying 't
  "Non-nil fixes ~ backup permissions on remote files created by ange-ftp."
  :group 'e-macs
  :type 'boolean)
(defcustom ange-ftp-make-backup-files 'nil
  "Nil fixes ~ backup permissions on remote files created by ange-ftp."
  :group 'e-macs
  :type 'boolean)

;;__________________________________________________________________________
;;;;;;		Web Browsing
;;w3 web browser from http://www.cs.indiana.edu/elisp/w3/docs.html
;;(load-library "w3-auto")

;;See inet.el, where we define browse-url-at-mouse() to launch <url:...>
(global-set-key [S-mouse-2] 'browse-url-at-mouse)

;;__________________________________________________________________________
;;;; APPLICATIONS
;;__________________________________________________________________________
;;;;;;		Babel Translation
;;Use babel fish web site to translate languages
;;These may get moved to i18n.el in the future...
;;This only works with Gnu Emacs
(defvar cache-babel)			;shut up compiler
(cache-locate-library
 use-cache 'cache-babel "babel"
 "For language translation support, get babel.el from <URL:http://www.chez.com/emarsden/downloads/babel.el>")

(autoload 'babel-mode "babel" "Babel translator." t)
(when cache-babel
  (progn
    (autoload 'babel "babel" "Babel translator." t)
    (autoload 'babel-region "babel" "Babel translator." t)
    (autoload 'babel-as-string "babel" "Babel translator." t)
    (autoload 'babel-buffer "babel" "Babel translator." t)
    (autoload 'babel-as-string "babel" "Babel translator." t)
    (defvar mark-active)                ;shut up bytecompiler

    (defun translate-with-babel (&optional arg)
      "Translates the word at point (if no region active) or whole region
	with babel. If ARG then the word at point (if no region active) or
	whole region is automatically replaced with the translation returned
	by babel. If ARG is nil then the translation-result is only displayed:
	If a region is active result is displayed in the standard babel-buffer
	otherwise result is displayed in the echo area."
      (interactive "P")
      (let* ((region-p (and mark-active (/= (mark) (point))))
             (babel-result
              (if region-p
                  (babel-as-string (buffer-substring-no-properties
                                    (mark) (point)))
                (babel-as-string (thing-at-point 'word))))
             ;; do additonal washing cause of some babel-backends return
             ;; space-stuff in front of the translation
             (index (string-match "[^ 	\n\f].*" babel-result)))
        (if index (setq babel-result (substring babel-result index)))
        (if (not arg)
            ;; no replacing only displaying the translation
            (if region-p
                (with-output-to-temp-buffer "*babel*"
                  (princ babel-result)
                  (save-excursion
                    (set-buffer "*babel*")
                    (babel-mode)))
              (message babel-result))
          ;; translation replaces the original text.
          (if region-p
              (kill-region (mark) (point))
            (autoload 'beginning-of-thing "thingatpt" "Thingatpt." t)
            (beginning-of-thing 'word)
            (kill-word 1))
          (insert babel-result))))

    (defadvice babel (after babel-advice activate)
      "Because the buffer \" *babelurl*\" is always modified we kill it after
     finishing babel so functions like save-some-buffers,
     save-buffers-kill-emacs ... dont annoying us with confirmation requests"
      (let ((buf-name " *babelurl*"))
        (if (member buf-name (mapcar (function buffer-name) (buffer-list)))
            (save-excursion
              (set-buffer buf-name)
              (set-buffer-modified-p nil)
              (kill-buffer buf-name)))))
    ))

;;__________________________________________________________________________
;;;;;;		Telephone/Address Book Function
;;X Emacs version does not provide (goto-address)
(global-set-key [(meta f3)] 'address-book)
(defun address-book ()
  "Opens .addressbook with goto address support."
  (interactive)  (find-file "~/emacs/addressbook") (goto-address))

;;__________________________________________________________________________
;;;; PROGRAMMING

;;__________________________________________________________________________
;;;;;;		Emacs Lisp
;;Enable compile from menu. Assumes emacs is in the path.
;;Note that starting another copy of emacs is not efficient, but at least
;;the menu does something relevant.
(add-hook
 'emacs-lisp-mode-hook
 (function
  (lambda ()
    (set (make-local-variable 'compile-command)
         (format "emacs -batch -f batch-byte-compile %s" buffer-file-name)))))

;;__________________________________________________________________________
;;;;;;		Function Menu
;;Jump to function from menu
(require 'imenu)
;;must be at least v20.3
;; (add-hook 'c-mode-hook (function (lambda () (imenu-add-to-menubar 'Func))))
(add-hook 'emacs-lisp-mode-hook (function (lambda ()
					    (imenu-add-to-menubar 'Func))))

;;Make imenu a mouse menu popup:
(define-key global-map [M-S-down-mouse-3] 'imenu)

;;__________________________________________________________________________
;;;;;;		Tags: Find C Declarations & References Fast
;;note: this section need to be before the C language preference so that
;;my-C-find-tag-default can piggy-back off of etags' find-tag-default
;; tweak tags library so that syntax is chosen from first file in tags file
;;(require 'etags)


;; (autoload 'etags "etags" "find variable declarations." t)


;;This section has byte-compile problems on v20.2
;;(autoload 'find-tag-default "etags" "find variable declarations." t)

;;(defun my-C-find-tag-default ()
;;  "If on a #include line, use the included file as the tag."
;;  (or (save-excursion
;;        (beginning-of-line)
;;        (if (looking-at
;;             "^[ \\t]*#[ \\t]*include[ \\t]*[<\"]\\([^>\"]+\\)[>\"]")
;;            (match-string 1)))
;;      (find-tag-default)))

;;(defvar tags-table-format-hooks)

;;(defconst librarys-recognize-table 
;;      (car tags-table-format-hooks))
;;(setcar tags-table-format-hooks
;;        '(lambda ()     "DOCUMENT ME!"
;;           (if (funcall librarys-recognize-table)
;;               (set-syntax-table
;;                (save-excursion
;;                  (goto-char (point-min))
;;                  (search-forward "\f\n")
;;                  (set-buffer (find-file-noselect (file-of-tag)))
;;                  (syntax-table)))
;;             nil)))

(defun re-search-forward-sensitive (REGEXP &optional BOUND NOERROR COUNT)
  "Wrap 're-search-forward'.
A local bind of 'case-fold-search' to nil now forces a case-sensitive search.
Argument REGEXP is a regular expression.
Optional argument BOUND DOCUMENT ME!
Optional argument NOERROR DOCUMENT ME!
Optional argument COUNT DOCUMENT ME!"
  (let ((case-fold-search nil))
    (re-search-forward REGEXP BOUND NOERROR COUNT)))

(defvar tags-loop-scan)
(defvar tags-loop-operate)
(defun my-tags-search (arg regexp &optional file-list-form)
  "Prefix ARG will force case-sensitivity.
Search through all files listed in tags table for match for REGEXP.
Stops when a match is found.  To continue searching for next match, use command
        \\[tags-loop-continue].
See documentation of variable 'tags-file-name'.
Optional argument FILE-LIST-FORM DOCUMENT ME!"
  (interactive "P\nsTags search (regexp): ")
  (let ((search-function (if arg 're-search-forward-sensitive
                           're-search-forward)))
    (if (and (equal regexp "")
             (eq (car tags-loop-scan) search-function)
             (null tags-loop-operate))
        ;; Continue last tags-search as if by M-,.
        (tags-loop-continue nil)
      (setq tags-loop-scan
            (list search-function (list 'quote regexp) nil t)
            tags-loop-operate nil)
      (tags-loop-continue (or file-list-form t)))))

(global-set-key "\C-x," 'my-tags-search)

;;__________________________________________________________________________
;;;;    Outline-mode control variables
;;4 ';'s at beginning of line makes an index entry
;;more ';'s nests the entry deeper

;; Local Variables:
;; mode: outline-minor
;; center-line-decoration-char: ?-
;; center-line-padding-length: 1
;; center-line-leader: ";;;; "
;; fill-column: 79
;; line-move-ignore-invisible: t
;; outline-regexp: ";;;;+"
;; page-delimiter: "^;;;;"
;; End:

(provide 'e-macs)

;;; e-macs.el ends here

